SUBROUTINE simcoda(station)
!---------------------------------------------------------------------
!
! Description:
!
!   Simulate S-wave coda for synthetic Green's function using Zeng's 
!   multiple S-to-S scattering theory
!
! Dependencies:
!
!   Subroutines fork, scoda, coda, ener2, evwi
!
! Fundamental input parameters:
!
!   nfcoda    - fft points for the coda envelope            m=mx
!   merr      - tolerance of the coda envelope              err=errx
!   abscoeff  - absorption coefficient                      gi=gix
!   scatcoeff - scattering coefficient  	                gs=gsx
!   fmax      - maximum frequency for coda waves   
!   Q         - Q0 attenuation factor {Q(f)=Q0*f**fdec}
!   fdec      - frequency decay of Q(f) factor	            cn=cnx
!   iseed     - seed number                                
!   nscat     - number of scattering wavelets	
!   hpass	  - highpass corner of the cosine filter        fl=flx
!   trans     - transition bandwidth of the filter          flb=flbx
!   t_len	  - length of time series                       twin=twinx
!   npts	  - number of points computed for coda envelope nt=ntx
!   t0	      - time of first arrival 
!   kappa     - kappa factor akp=akpx
!   sr_hypo   - station-hypocenter distance          	    dist=distx
!   aveVs	  - average S-speed between S-R                 vs=vsx
!   siteVs    - S-wave velocity at the site	                vs0=vs0x
!   srcVs     - S-wave velocity at the source               vs1=vs1x
!   siteR     - density at the site                         rh0=rh0x
!   srcR      - density at the source                       rh1=rh1x
!
! References:
!
!   Zeng, Anderson and Su (1995).  Subevent rake and random 
!   scattering effects in realistic strong ground motion 
!   simulation, Geophy. Res. Lett., 22 17-20
!
! Note:
!
!   Allmost all needed parameters are passed by modules
!
! Authors: W. Imperatori, M. Mai, Y. Zeng 
!
! Modified: January 2009 (v1.3)
!

use constants; use def_kind; use scattering; use source_receiver
use waveform, only: lf_len, scattgram

implicit none

! local variables
integer(kind=i_single),intent(in)       :: station 
complex(kind=r_single),dimension(npts)  :: u2
complex(kind=r_single),dimension(npts,3):: u1
complex(kind=r_single),dimension(npts*4):: u
real(kind=r_single)                     :: kappa_local,dt,df,dw,w,x
real(kind=r_single)                     :: c1,c2,c3,tmp,band,db,Qt
integer(kind=i_single)                  :: k,nw,nwm,j,l,ml,mlb
integer(kind=i_single)                  :: mmb,i

!---------------------------------------------------------------------

! compute some parameters
kappa_local=kappa(station)*0.5           !check this
dt=lf_len/float(npts-1)
df=1./lf_len                             
dw=pi*df


k=npts+2
nw=npts/2
nwm=fmax/df+1
if (nwm.gt.nw) nwm=nw
dw=dw+dw
w=-dw+0.00001

do j=1,nwm
   w=w+dw
   Qt=1.0/(Q*(w/pi_double)**fdec)
   u2(j)=((w-w*log(w/pi_double)*Qt/pi)/aveVs)*cmplx(0.5*Qt,1.0)
enddo

u1=zero

do l=1,nscat
   x=aveVs*(t0+random_array(1,l,station)*lf_len)
   c1=random_array(2,l,station)*2.0-1.0
   c2=random_array(3,l,station)*2.0-1.0
   c3=random_array(4,l,station)*2.0-1.0

   do j=1,nwm
      tmp=exp(-x*u2(j))
      u1(j,1)=u1(j,1)+c1*tmp
      u1(j,2)=u1(j,2)+c2*tmp
      u1(j,3)=u1(j,3)+c3*tmp
   enddo

enddo
                            
!  filter the low frequency
ml=ifix(hpass/df)+1
mlb=ifix(trans/df)
if(mlb.eq.0) mlb=1
mmb=ml-mlb+1
if(mmb.le.1) mmb=2
   
do i=1,3

   do j=nwm+1,nw+1
      u(j)=zero
      u(k-j)=zero
   enddo

   u(1)=zero

   do j=2,ml-mlb
      u(j)=zero
      u(k-j)=zero
   enddo

   band=-1.0
   db=1./float(mlb)
   w=(mmb-1)*dw

   do j=mmb,ml+mlb-1
      w=w+dw
      band=band+db
      c1=df*(0.5+0.5*sin(pi_half*band))
      u(j)=u1(j,i)*exp(cmplx(-w*kappa_local,w*t0))*c1
      u(k-j)=conjg(u(j))
   enddo
 
   do j=ml+mlb,nwm
      w=w+dw
      u(j)=u1(j,i)*exp(cmplx(-w*kappa_local,w*t0))*df
      u(k-j)=conjg(u(j))
   enddo

   call fork(npts,u,1.)
	
   do j=1,npts
      scattgram(j,i,station)=real(u(j))
   enddo

enddo  

dw=dw*0.5

! simulate strong motion coda waves for each subfaults
call scoda(sr_hypo(station),dt,station)


END SUBROUTINE simcoda

!=============================================================================

SUBROUTINE fork(lx,cx,signi)

use def_kind

implicit none

integer(kind=i_single),intent(in)                 :: lx
real(kind=r_single),intent(in)                    :: signi
complex(kind=r_single),dimension(lx),intent(inout):: cx

!local
complex(kind=r_single):: carg,cw,ctemp
integer(kind=i_single):: j,i,m,l,istep

j=1

do i=1,lx

   if(i.le.j) then
      ctemp=cx(j)
      cx(j)=cx(i)
      cx(i)=ctemp
   end if

   m=lx/2 
20 if(j.le.m) goto 30
   j=j-m
   m=m/2 
   if(m.ge.1) goto 20
30 j=j+m

enddo

l=1 
40 istep=2*l

do m=1,l
   carg=(0.,1.)*(3.14159265*signi*float(m-1))/float(l)
   cw=cexp(carg)

   do i=m,lx,istep
      ctemp=cw*cx(i+l) 
      cx(i+l)=cx(i)-ctemp
      cx(i)=cx(i)+ctemp 
   enddo

enddo

l=istep
if(l.lt.lx) goto 40

END SUBROUTINE fork

!=============================================================================

SUBROUTINE scoda(dist,dt,station)

!  Program : scoda.f
!  Purpose : simulate strong motion S-wave coda
!                                by Yuehua Zeng at UNR, Oct 1993

use constants; use def_kind; use scattering
use waveform, only: lf_len, scattgram

implicit none

real(kind=r_single),intent(in):: dist,dt
integer(kind=i_single),intent(in):: station

!local
real(kind=r_single),dimension(npts):: p,t
real(kind=r_single):: c0,c1,dt1,t1,pp
integer(kind=i_single):: i1,i2,ii,i,i0,n1

!	parameter (mm1=8192,mm2=800)
!	dimension pcoda(mm1,3),t(mm2),p(mm2)
!	common/codaw/n,m,err,gi,gs,fl,flb,rh0,rh1,vs0,vs1,vs,twin,dt
!	pi=3.1415926

!  generate the coda waves
call coda(2.0*lf_len,dist,t,p,dt)

 c0=0.85/(pi*4.*dist*aveVs)*sqrt(srcVs*srcR/(siteVs(station)*siteR(station)))
 c1=sqrt(3.0*lf_len/float(nscat))*c0
dt1=t(2)-t(1)
i1=ifix(t0/dt+0.000001)
i2=ifix(t(1)/dt+0.000001)

do ii=1,3
   do i=1,ncoda
      if(t0.le.t(i))goto 65
   enddo
 
65 n1=i
   i0=0
   t1=float(i1-1)*dt
 
   do i=1,i2-i1+1
      i0=i0+1
      t1=t1+dt
      scattgram(i,ii,station)=0.0
   enddo
	
   do i=i0+1,npts
      t1=t1+dt
      if(t1.gt.t(n1))n1=n1+1
      pp=p(n1-1)+(p(n1)-p(n1-1))*(t1-t(n1-1))/dt1
      scattgram(i,ii,station)=pp*(c1*scattgram(i,ii,station))
   enddo

enddo

END SUBROUTINE scoda

!=============================================================================

SUBROUTINE coda(tw,r,t,p,ddt)

use constants; use def_kind; use scattering

implicit none

real(kind=r_single),intent(in)                 :: tw,r
real(kind=r_single),dimension(npts),intent(out):: p,t
real(kind=r_single),intent(in)                 :: ddt

!  Program for calculating the scattered wave energy using equation
!  (24) of Zeng et al. (1991).  Written by Yuehua Zeng in May 1990
!  and updated by Yuehua Zeng on Feb. 9, 1993.
!------------------------------------------------

! local variables
complex(kind=r_single):: atinv,atinv1,atinv3,akw,wg
complex(kind=r_single):: ei2,aker,Es,Es1,Es2
complex(kind=r_single),dimension(1024):: au

real(kind=r_single):: ep,g,dw,dk,dt,w,ak,sinkr,wi
real(kind=r_single):: c0,c1,c2,c3,gamma,r2,vt,tau
real(kind=r_single):: pd,di,coef

integer(kind=i_single):: nd,ni,m2,i,nk,ik,j,n1

!	subroutine coda(n,m,gs,gi,vs,tw,r,err,t,p,ddt)
!	dimension t(n),p(n)
!	complex au(1024),atinv,atinv1,atinv3,akw,wg,one,zero,ei,
!     +		ei2,aker,Es,Es1,Es2
!
!  compute parameters
!	pi=3.14159265
!	one=cmplx(1.0,0.0)
!	zero=cmplx(0.0,0.0)

ei2=0.5/zeta
ep=1.0e-20
g=scatcoeff+abscoeff
dw=2.0*pi/tw
nd=2*ifix(0.25*(r+aveVs*tw)/r+1)
dk=pi/(r*float(nd))
ni=ifix(20.0/nd+1)*nd+nd/2
m2=nfcoda/2
dt=(tw-r/(aveVs*sqrt(3.0)))/float(ncoda)
t(1)=r/(aveVs*sqrt(3.0))

do i=2,ncoda
   t(i)=t(i-1)+dt
enddo

i=ifix((r/aveVs-t(1))/dt)+1
t(i+1)=r/aveVs+ddt
t(i)=r/aveVs-ddt

!  calculate the power spectrum for scattering order higher than two
call evwi

w=-dw

do i=1,m2
   w=w+dw
   wg=cmplx(g+wi/aveVs,w/aveVs)
   ak=0.0
   Es=zero
   Es1=zero
   Es2=zero
   nk=ni
   ik=0

   do j=1,10000
      ak=ak+dk
      akw=cmplx(0.0,ak)/wg
      atinv=ei2*clog((one+akw)/(one-akw))
      atinv1=scatcoeff*atinv/ak
      atinv3=atinv1*atinv1*atinv1
      aker=atinv3*atinv/(one-atinv1)
      sinkr=sin(ak*r)
      Es=Es+sinkr*aker
      if(cabs(aker/(Es+ep)).lt.merr.and.j.gt.20) goto 25
      if(j.eq.nk)then
         ik=ik+1
         Es2=Es2+Es-sinkr*aker*0.5
         if(ik.eq.6)then
            if(cabs((Es2-Es1)/(Es2+ep)).lt.merr)then
	       Es=(Es1+Es2)/12.0
	       goto 25
	    endif
	    ik=0
	    Es1=Es2
	    Es2=zero
	 endif
	 nk=nk+nd
      endif
   enddo

25 au(i)=Es*dk

enddo

n1=nfcoda+2
	
do i=1,m2
   au(n1-i)=conjg(au(i))
enddo

au(m2+1)=zero

call fork(nfcoda,au,1.0)

w=0.0
dw=wi*tw/float(nfcoda-1)

do i=2,nfcoda
   w=w+dw
   au(i)=au(i)*exp(w)
enddo

!  loop for calculating time domain scattering energy decay
dt=tw/float(nfcoda)
 c0=0.25/(aveVs*pi*r*r)
 c1=0.25*scatcoeff/(aveVs*r*pi*c0)
 c2=scatcoeff*scatcoeff/(16.0*r*pi*c0)
 c3=0.5/(aveVs*r*pi*pi*tw*c0)
gamma=2.0/(9.0*sqrt(3.0))
r2=r*r

do i=1,ncoda
   vt=aveVs*t(i)
   if(vt.lt.r)then
      tau=exp(-g*vt)
      p(i)=gamma*c1*alog((vt+r)/(r-vt))/t(i)*tau
      goto 55
   endif
   tau=r/vt
 
   call ener2(0.01)

   n1=ifix(t(i)/dt)+1
   pd=(real(au(n1))+(real(au(n1+1))-real(au(n1)))  &
      *(t(i)-dt*float(n1-1))/dt)*exp(abscoeff*vt)
   tau=exp(-scatcoeff*vt)
   p(i)=c1*alog((vt+r)/(vt-r))/t(i)*tau+c2*coef*tau+c3*pd
55 p(i)=sqrt(p(i))
enddo
	
!..........................................................................

CONTAINS

   SUBROUTINE ener2(dd)
   !  calculate coefficient of the second-order scattered energy term

   implicit none

   real(kind=r_single),intent(in):: dd
   !local
   integer(kind=i_single)        :: n,i
   real(kind=r_single)           :: dt,t,aux


   n=ifix(tau/dd)+1
   dt=tau/float(n)
   coef=0.0
   t=0.0

   do i=1,n
      t=t+dt
      aux=alog((1.0+t)/(1.0-t))
      aux=aux*aux
      coef=coef+aux
   enddo

   coef=3.0*(0.5*aux-coef)*dt
   coef=coef+tau*pi*pi

   END SUBROUTINE ener2
!.....................................................

   SUBROUTINE evwi
   !  estimating the value of imaginery frequency wi

   implicit none

   real(kind=r_single)   :: dd,vt,ud,du,coef,u
   real(kind=r_single)   :: aux,di,dimax
   integer(kind=i_single):: i,m,j 


   dd=0.01
   vt=r

   do i=1,2
      vt=vt+(aveVs*tw-r)*(i-1)
      ud=sqrt(0.75*scatcoeff*vt)
      m=ifix(ud/dd)+1
      du=ud/float(m)
      coef=0.5
      u=0.0

      do j=1,m
         u=u+du
         aux=exp(-u*u)
         coef=coef+aux
      enddo
	
      coef=(coef-0.5*aux)*du-ud*exp(-ud*ud)
      u=0.75*scatcoeff/vt
      aux=(1.-exp(-scatcoeff*vt)*(1.+scatcoeff*vt))/coef
      di=aux*exp(1.5*alog(u/real(pi))-u*r*r-abscoeff*vt)
 
      if(i.eq.1)then
         dimax=di
      end if

   enddo

   wi=alog(500*di/dimax)/tw

   END SUBROUTINE evwi 

END SUBROUTINE coda