SUBROUTINE broadband_comp(station)
!------------------------------------------------------------------------------------
!
! Description:
!
!   External subroutine, calculates the composite broad-band seismograms, given the 
!   low and high frequency time-series. The waveform data are assumed to be velocity
!   seismograms [m/sec] for the three components of motion x, y, and z. It operates
!   in the frequency domain using the approach of Mai and Beroza.
!
! Dependencies:
!
!   Subroutine bb_calc  
!
! References:
!
!   Numerical Recipes
!
! Authors: W. Imperatori, M. Mai, B. Mena
!
! Modified: January 2009 (v1.3)
!

use constants; use def_kind; use flags; use scattering, only: npts
use source_receiver; use waveform

implicit none

! actual station number
integer(kind=i_single),intent(in)             :: station
! indexes, taper length
integer(kind=i_single)                        :: i,ind,taper_len
! time-step
real(kind=r_single)                           :: dt
! taper
real(kind=r_single),allocatable,dimension(:,:):: taper

!------------------------------------------------------------------------------------

! delta-t of both high and low-frequency (already interpolated) time-series   
dt = lf_len/(npts-1)

! calculate broadband seismograms for each component
do i=1,3   
   call bb_calc(i,dt,station)
enddo
   
if (modality_flag /= 0) then   
   ! find time-index corresponding to P-wave arrival 
   ind=nint(time_p(station)/dt)                        

   ! setting up the taper (rising half of Hanning window) to avoid spikes in acceleration 
   ! at P-wave onset
   ! taper length in points (this is the effective length)
   taper_len=10

   ! reduce taper if too long
   if (floor(taper_len/2.) >= ind) then
      taper_len = 2 * ind - 2 
      call warning_handling(1,'null','BROADBAND_COMP (composition.f90)') 
   endif

   ! allocate local array for taper
   if (.not.allocated(taper)) allocate(taper(taper_len,3))

   ! equation of Hanning window: h(i)=0.5*(1-cos(2*pi*i/N)) ---> see Numerical Recipes, page: 547
   ! computing only the first (rising) half
   do i=1,taper_len
      taper(i,:) = 0.5 * (1 - cos(2 * pi * i / (2*taper_len)) )
   enddo

   ! set to zero all amplitudes values about before the P-wave arrival and apply taper
   bb_seis(1:ind-floor(taper_len/2.),:,station)=0.0 
   bb_seis(ind-floor(taper_len/2.)+1:ind+ceiling(taper_len/2.),:,station) =     &
   bb_seis(ind-floor(taper_len/2.)+1:ind+ceiling(taper_len/2.),:,station)*taper
   
   ! deallocate memory
   if (allocated(taper)) deallocate(taper)   

endif

END SUBROUTINE broadband_comp

!===================================================================================================

SUBROUTINE bb_calc(comp,dt,station)
!-----------------------------------------------------------------------------------
!
! Description:
!
!   External subroutine, merges the HF and LF seismograms into broadband time-series
!   using the approach of Mai and Beroza
!
! Dependencies:
!
!   Subroutine fast_fourier
!
! Notes:
!
!   Input time-series MUST have the same number of points and time-step. The FFT/IFFT
!   subruotine implemented works only with 2**n time-series. 
!   Frequency vector goes from 0 to f_nyq and back to 0+df (npts points)
!
! References:
!
!
!
! Authors: W. Imperatori, M. Mai, B. Mena
!
! Modified: January 2009 (v1.3)
!

use interfaces, only: four1d
use constants; use def_kind; use matching
use scattering, only: npts; use waveform

implicit none

! current component and station number
integer(kind=i_single),intent(in)           :: comp,station
! time-step
real(kind=r_single),intent(in)              :: dt
! indexes, counters, dummies 
integer(kind=i_single)                      :: index_diff,index_min,fscale_1,fscale_2,acc_size
! flag and counters 
integer(kind=i_single)                      :: trasf_sign,i
! index of real target frequency and Nyquist frequency 
integer(kind=i_single)                      :: targ_index,f_nyq_index
! window for search: half extension, lower and upper bounds, total extension 
integer(kind=i_single)                      :: win_fr,win_low,win_up,win_npts
! array for complex time-series
complex(kind=r_single),dimension(npts)      :: hf_four,lf_four,broad_complex
! arrays for high and low frequency spectra (amplitude and phase) 
real(kind=r_single),dimension(npts)         :: Am_hf,Ph_hf,Am_lf,Ph_lf
! arrays for scaling 
real(kind=r_single),dimension(npts/2 +1)    :: Acc_hf,Acc_lf
! frequency vectors, windowing functions, arrays for phase and amplitude of merged time-series
real(kind=r_single),dimension(npts)         :: f,fn_lf,fn_hf,phase,amp
! arrays for windowed spectra (amplitude and phase) 
real(kind=r_single),dimension(npts)         :: hf_am_win,lf_am_win,hf_ph_win,lf_ph_win
! array for amplitude differences 
real(kind=r_single),allocatable,dimension(:):: diff_amp
! actual target frequency, averaged spectra and their ratio, Nyquist frequency
real(kind=r_single)                         :: f_targ,Av_hf,Av_lf,ratio,f_nyq
! delta-f
real(kind=r_single)                         :: df

!----------------------------------------------------------------------------------- 
	
! computing frequency-delta
df = 1 / (npts * dt)    !according to NR

! compute nyquist frequency (its index is npts/2 +1) 
! (since npts is multiple of 2, f_nyq = {(npts+2)/2 -1)*df}
f_nyq = 0.5 * (1 / dt)         
f_nyq_index = npts/2 + 1
 
! create frequency vector
f(1)=0.	
do i=1,f_nyq_index-1
   f(i+1)=f(i)+df   
enddo
do i=f_nyq_index,npts-1
   f(i+1)=f(i)-df
enddo   
   
! flag for FFT (i.e. from time to frequency)
trasf_sign=1    
	
hf_four=cmplx(conv_seis(:,comp,station),0.0)   !prepare complex array for HF

! calling function for FFT
call four1d(hf_four,trasf_sign)

! extracts Amplitude and Phase spectra (HF)
Am_hf=cabs(hf_four)
Ph_hf=atan2(aimag(hf_four),real(hf_four))

lf_four=cmplx(lf_int(:,comp),0.0)   !prepare complex array for LF	

! calling function for FFT
call four1d(lf_four,trasf_sign)

! extracts Amplitude and Phase spectra (LF)
Am_lf=cabs(lf_four)
Ph_lf=atan2(aimag(lf_four),real(lf_four))

! index of target frequency in frequency vector 
do i=1,f_nyq_index
   if (f(i) >= targ_fr-df .and. f(i) <= targ_fr+df) then
      targ_index=i
      exit
   endif   
enddo   

! actual target frequency (user target-fr. "projected" onto frequency vector)
f_targ=f(targ_index)   

! search window half-length (number of points in frequency)	
win_fr = nint(band_wid/df)

! define search region on frequency vector (in points)
win_low = targ_index - win_fr
win_up = targ_index + win_fr
win_npts = (win_up-win_low) + 1      !total window-length for search (i.e. search area)

! check for search window 
if ((win_low < 1) .or. (win_up > f_nyq_index)) then 
   call error_handling(6,'0','BB_CALC (composition.f90)')
endif

! allocating vector where to store amplitude differences
if (.not.allocated(diff_amp)) allocate(diff_amp(win_npts))

! computes spectral amplitude differences inside the search area
diff_amp(1:win_npts)=abs(Am_hf(win_low:win_up)-Am_lf(win_low:win_up))

! find index of minimum inside amplitude-difference vector	
index_diff=minloc(diff_amp,dim=1) 

! deallocate memory 
if (allocated(diff_amp)) deallocate(diff_amp)

! find index of minimum inside positive frequency vector  
index_min=index_diff-1+win_low    

! determine matching frequency, which corresponds to the minimum amplitude difference
match_fr(comp,station)=f(index_min)


! ----> section for AMPLITUDE SCALING (to avoid spectral jumps at the matching frequency) <----

! scaling is based on computing the ratio of LF/HF acceleration amplitude spectra 
! averaged over a wide window

! derive acceleration amplitude spectra from velocity amplitude spectra 
Acc_hf=Am_hf(1:f_nyq_index)*2.*pi*f(1:f_nyq_index) 
Acc_lf=Am_lf(1:f_nyq_index)*2.*pi*f(1:f_nyq_index)
! note: here above the problem WAS that f went from -f_nyq to f_nyq, while Am from 0 to 2*fnyq-1 

! compute spectral averages over a large window (3 times search window)
fscale_1=index_min-3*win_fr; if (fscale_1 <= 0) fscale_1=1
fscale_2=index_min+3*win_fr; if (fscale_2 > f_nyq_index) fscale_2=f_nyq_index
acc_size=(fscale_2-fscale_1)+1

! check for the above secondary search window
!!if ( (fscale_1 == 1) .or. (fscale_2 == f_nyq_index) ) then
!!   call warning_handling(2,'0','BB_CALC (composition.f90)')
!!endif

! compute averages
Av_hf = sum(Acc_hf(fscale_1:fscale_2))/(acc_size-1)   
Av_lf = sum(Acc_lf(fscale_1:fscale_2))/(acc_size-1) 

ratio = Av_lf/Av_hf   !averages ratio

! scale the HF seismogram by ratio
if (ratio > 2.0) Am_hf=Am_hf*ratio*0.5  
if ((1./ratio) > 2.0) Am_hf=Am_hf*ratio*1.5 

! ----------------------> end section for amplitude scaling <---------------------- 

! windowing function for low frequency
where ( f <= match_fr(comp,station) )
   fn_lf=1.0
elsewhere
   fn_lf=0.
end where

! windowing function for high frequency 
where ( f > match_fr(comp,station) )
   fn_hf=1.0
elsewhere
   fn_hf=0.
end where

! multiply phase and amplitude spectra with the windowing function
hf_ph_win=Ph_hf*fn_hf; lf_ph_win=Ph_lf*fn_lf
hf_am_win=Am_hf*fn_hf; lf_am_win=Am_lf*fn_lf      

! compose low and high frequency phase spectra	
phase=hf_ph_win+lf_ph_win	

! compose low and high frequency amplitude spectra
amp=hf_am_win+lf_am_win

! combine amplitude and phase spectra (whole broadband spectrum) 
broad_complex=amp*exp(zeta*phase)	

! perform IFFT to find time history from broadband spectrum
trasf_sign=-1
call four1d(broad_complex,trasf_sign)

! output broadband time-series (after scaling)
bb_seis(:,comp,station)=real(broad_complex/npts)	


END SUBROUTINE bb_calc
