MODULE def_kind
!
! Definition:
!
!   This is to guarantee portability on different machines. P indicates "precision"
!
! Dependencies:
!
!   All subroutines and modules
!
! Author: W. Imperatori
!
! Modified: January 2009 (v1.3)
!

implicit none

! Real single precision
integer,parameter:: r_single=selected_real_kind(p=6)
! Real double precision
integer,parameter:: r_double=selected_real_kind(p=15)
! Real 'scattering' precision (i.e. only 4 digits after decimal point)
integer,parameter:: r_scat=selected_real_kind(p=4)    
! Integer single representation (i.e. up to 10**9 order)
integer,parameter:: i_single=selected_int_kind(9)  
! Integer double representation (i.e. up to 10**18 order)
integer,parameter:: i_double=selected_int_kind(18)

END MODULE def_kind

!===================================================================================================

MODULE constants
!
! Definition:
!
!   Define several constants used in the code
!
! Dependencies:
!
!   Module def_kind
!
! Author: W. Imperatori
!
! Modified: January 2009 (v1.3)
!

use def_kind

implicit none

! Pi and 2*Pi
real(kind=r_double),parameter   :: pi=3.141592653589793, pi_double=3.141592653589793*2.
! Pi/2 and degree-to-radiant conversion factor
real(kind=r_double),parameter   :: pi_half=3.141592653589793/2., d2r=3.141592653589793/180.
! Complex triplet
complex(kind=r_single),parameter:: zero=cmplx(0.,0.), one=cmplx(1.0,0.0), zeta=cmplx(0.0,1.0)

END MODULE constants

!===================================================================================================

MODULE earthquake
!
! Description:
!
!   Contains event parameters  
!
! Dependencies:
!
!   Module def_kind
!
! Author: W. Imperatori
!
! Modified: January 2009 (v1.3)
!

use def_kind

implicit none

save

! Magnitude of the event (assigned via main input file)
real(kind=r_single):: Mw
! Mechanism of the event (assigned via main input file)
character(len=2)   :: mech


END MODULE earthquake

!===================================================================================================

MODULE flags
!
! Description:
!
!   Various flags 
!
! Dependencies:
!
!   Module def_kind.
!
! Author: W. Imperatori
!
! Modified: January 2009 (v1.3)
!

use def_kind

implicit none

save

! Flags used in the main input file (velocity model, extended fault, modality flag)
integer(kind=i_single):: vel_flag,ext_flag,modality_flag
! Flag for secondary output (log-file) and LF-HF files format
character(len=4)      :: verbose_flag,lf_kind_flag,hf_kind_flag


END MODULE flags

!===================================================================================================

MODULE geometry
!
! Description:
!
!   Store parameters regarding spatial domain and its discretization      
!
! Dependencies:
!
!   Module def_kind
!
! Author: W. Imperatori
!
! Modified: January 2009 (v1.3)
!

use def_kind

implicit none

save

! Dimensions and grid-spacing of the computational domain (defined in main input file)
real(kind=r_single)                         :: h_step,x_far,y_far,z_far,x_init,y_init,z_init 
! Hypocenter coordinates
real(kind=r_single)                         :: hyp_x,hyp_y,hyp_z 
! Number of layers for 1D model (if provided)
integer(kind=i_single)                      :: n_lay    
! Number of grid-points for raytracing code
integer(kind=i_single)                      :: nx,ny,nz
! Arrays for the raytracing code
integer(kind=i_single),dimension(3)         :: grid
real(kind=r_single),dimension(3)            :: hypo
! Array for delay values
real(kind=r_single),allocatable,dimension(:):: tau


END MODULE geometry

!===================================================================================================

MODULE io_file
!
! Description:
!
!   Store values related to input/output operations
!
! Dependencies:
!
!   Module def_kind
!
! Author: W. Imperatori
!
! Modified: January 2009 (v1.3)
!

use def_kind

implicit none

save
 
! Secondary input files (velocity model, stations, extended fault and scattering)
character(len=90):: vel_file,stat_file,ext_file,scat_file
! Optional: 2nd station file and binary input file for HF/LF waveforms
character(len=90):: opt_stat_file,lf_bin_file,hf_bin_file   
! time-series file extensions 
character(len=4) :: lf_x,lf_y,lf_z,hf_x,hf_y,hf_z
! input LF directory and optional (HF) directory for already computed HF files
character(len=90):: lf_in_dir,opt_dir,output_dir 

END MODULE io_file

!===================================================================================================

MODULE matching
!
! Description:
!
!   Contains parameters for LF-HF matching operations  
!
! Dependencies:
!
!   Module def_kind
!
! Author: W. Imperatori
!
! Modified: January 2009 (v1.3)
!

use def_kind

implicit none

save

! Target frequency and bandwidth (assigned via main input file)
real(kind=r_single)                           :: targ_fr,band_wid
! Actual matching frequency (in the range targ_fr +/- band_wid)
real(kind=r_single),allocatable,dimension(:,:):: match_fr     

END MODULE matching

!===================================================================================================

MODULE scattering
!
! Description:
!
!   Store scattering-related parameters.
!
! Dependencies:
!
!   Module def_kind.
!
! Notes:
!
!   For a complete description of many parameters, see coda.f90
!
! Author: W. Imperatori
!
! Modified: January 2009 (v1.3)
!

use def_kind

implicit none

save

! The following parameters MAY change for every receiver
! Average S-wave speed
real(kind=r_scat)                         :: aveVs
! Site-specific rho, Vs and kappa
real(kind=r_scat),allocatable,dimension(:):: siteR,siteVs,kappa  
! Seed number used to compute scattered wavelets
integer(kind=i_single)                    :: iseed 

! The following parameters DON'T change between receivers and can be changed by the user
! Source rho and Vs 
real(kind=r_scat)                         :: srcR,srcVs   
! Coda envelope and fft points, scattering wavelets
integer(kind=i_single)                    :: ncoda,nfcoda,nscat
! Seed number used to (optionally) compute some medium scattering properties
integer(kind=i_single)                    :: s_seed
! Highpass corner, transition bandwidth, coda envelope tolerance, absorption coefficient 
real(kind=r_scat)                         :: hpass,trans,merr,abscoeff               
! Scattering coefficient, frequency decay, attenuation at f0
real(kind=r_scat)                         :: scatcoeff,fdec,Q                  
    
! The following parameters are FIXED and should NOT be changed
! Maximum frequency and start-time of scattered wavelets
real(kind=r_scat),parameter               :: fmax=20.,t0=0.       
! Time-domain points of scattered wavelet
integer(kind=i_single)                    :: npts                         

! Random numbers array, necessary to guarantee the same results 
! between serial and parallelized executions
real(kind=i_single),allocatable,dimension(:,:,:):: random_array

!$OMP THREADPRIVATE(aveVs)

END MODULE scattering

!===================================================================================================

MODULE source_receiver
!
! Description:
!
!   Store different source & stations parameters (X-Y coordinates already shifted, 
!   see io.f90)
!
! Dependencies: 
!
!   Module def_kind
!
! Author: W. Imperatori
!
! Modified: January 2009 (v1.3)
!

use def_kind

implicit none

save

! Contains mandatory and optional stations names
character(len=90),allocatable,dimension(:)    :: stat_name, opt_stat_name 
! Number of stations and extended-source cells
integer(kind=i_single)                        :: n_stat,n_cell
! Stations coordinates               
real(kind=r_single),allocatable,dimension(:)  :: xp,yp      
! Extended-source cells coordinates
real(kind=r_single),allocatable,dimension(:)  :: x_cell,y_cell,z_cell  
! S&P-waves traveltimes, hypocentral distances
real(kind=r_single),allocatable,dimension(:)  :: time_s,time_p,sr_hypo   
! Source-receiver distances for each subfault 
real(kind=r_single),allocatable,dimension(:,:):: sr_cell      

END MODULE source_receiver

!===================================================================================================

MODULE stf_data
!
! Description:
!
!   Store Source-Time-Function parameters
!
! Dependencies: 
!
!   Module def_kind
!
! Author: W. Imperatori
!
! Modified: January 2009 (v1.3)
!

use def_kind

implicit none

save

! Kind of source-time-function selected (from main input file)
character(len=5)                            :: stf_name
! Number of time-points for the source-time-function 
integer(kind=i_single)                      :: npts_stf
! Source-time-function (amplitude values) and its time vector
real(kind=r_single),allocatable,dimension(:):: stf ,t_stf
! Tapering window
real(kind=r_single),allocatable,dimension(:):: han_win
! Source-time-function duration and its starting point (in sec.) 
real(kind=r_single)                         :: total,ton                          
                                            
END MODULE stf_data

!===================================================================================================

MODULE waveform
!
! Description:
!
!   Store LF and HF time-series and some related parameters
!
! Dependencies: 
!
!   Module def_kind   
!
! Author: W. Imperatori
!
! Modified: January 2009 (v1.3)
!

use def_kind

implicit none

save

! LF and (optional) HF waveforms
real(kind=r_single),allocatable,dimension(:,:,:):: lf_seis,hf_seis
! Scatterograms (Coda waves) 
real(kind=r_single),allocatable,dimension(:,:,:):: scattgram
! Scatterograms convolved with source-time-function
real(kind=r_single),allocatable,dimension(:,:,:):: conv_seis 
! Final broad-band seismograms 
real(kind=r_single),allocatable,dimension(:,:,:):: bb_seis   

! Number of points for LF seismograms 
integer(kind=i_single)                          :: lf_npts                                         
! LF time-series length 
real(kind=r_single)                             :: lf_len 
! LF waveforms interpolated (to avoid spectral aliasing up to fmax)
real(kind=r_single),allocatable,dimension(:,:)  :: lf_int          

! Number of points for already computed HF seismograms 
integer(kind=i_single)                          :: hf_npts                                         
! HF time-series length 
real(kind=r_single)                             :: hf_len 
    

!$OMP THREADPRIVATE(lf_int)



END MODULE waveform

!===================================================================================================